"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventSourceParserStream = void 0;
const parse_js_1 = require("./parse.cjs");
/**
 * A TransformStream that ingests a stream of strings and produces a stream of ParsedEvents.
 *
 * @example
 * ```
 * const eventStream =
 *   response.body
 *     .pipeThrough(new TextDecoderStream())
 *     .pipeThrough(new EventSourceParserStream())
 * ```
 * @public
 */
class EventSourceParserStream extends TransformStream {
    constructor() {
        let parser;
        super({
            start(controller) {
                parser = (0, parse_js_1.createParser)((event) => {
                    if (event.type === "event") {
                        controller.enqueue(event);
                    }
                });
            },
            transform(chunk) {
                parser.feed(chunk);
            },
        });
    }
}
exports.EventSourceParserStream = EventSourceParserStream;
